#!/bin/bash
#

# buildRoot.sh

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e14|e14:nu|e13|e13:nu|e10|e10:nu> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_local_qualifiers()
{

    if [[ "${basequal}" == nu:* ]]; then
      # Reorder quietly.
      basequal="${basequal#nu:}:nu"
    fi

    if [[ "${basequal}" == *:nu ]]; then
      shortqual=nu
    else
      shortqual=$(echo "${basequal}" | cut -c 1)
    fi

    cqual="${basequal%%:nu}"
}

set_compiler_flags()
{
  case ${cqual} in
    e1[024])
      xcc=gcc
      local cc=gcc
      local cxx=g++
      local fc=gfortran
      ;;
    e13)
      xcc=clang
      local cc=clang
      local cxx=clang++
      local fc=gfortran
      ;;
    *)
      echo "ERROR: unrecognized compiler qualifier ${cqual}"
      usage
      exit 1
  esac
  compiler_specs=(-DCMAKE_C_COMPILER=${cc} \
      -DCMAKE_CXX_COMPILER=${cxx} \
      -DCMAKE_Fortran_COMPILER=${fc}
      )
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=root
origpkgver=v6_10_04
pkgver=${origpkgver}a
ssibuildshims_version=v0_19_00
pkgtarver=`echo ${origpkgver} | sed -e 's/_/./g'`
pkgdotver=`echo ${pkgtarver} | sed -e 's/v//'`
mmdotver=`echo ${pkgdotver} | cut -d. -f1-2`
# Downloaded tarball.
pkgtarfile=root_${pkgtarver}.source.tar.gz

get_this_dir
get_ssibuildshims
set_local_qualifiers

source define_basics --

if ! [[ -f "${tardir}/${pkgtarfile}" ]]; then
  # Git archive.
  pkgtarfile=root-${pkgdotver}.tar
fi

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

case ${extraqual} in
debug) buildtype=Debug;;
prof)  buildtype=MinSizeRel;;
opt)   buildtype=Release;;
* )
   echo "ERROR: please specify debug, opt, or prof"
   usage
   exit 1
;;
esac

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
# remove existing sourcedir to avoid patch conflicts
rm -rf ${sourcedir}
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup root
setup -B ${package} ${pkgver} -q +${fullqual//:/:+} -z ${fakedb}:${product_dir}:${PRODUCTS} || \
 ssi_die "fake setup failed: unsupported qualifier?"

# We need cmake
source ${SSIBUILDSHIMS_DIR}/bin/setup_cmake "${product_dir}" || ssi_die "failed to setup cmake"

if [ -z ${FFTW_INC} ]
then
   echo "ERROR: failed to setup fftw"
   exit 1
else
   echo "FFTW_INC: ${FFTW_INC}"
fi

if [ -z ${LIBXML2_INC} ]
then
   echo "ERROR: failed to setup libxml"
   exit 1
else
   echo "LIBXML2_INC: ${LIBXML2_INC}"
fi

if [ -z ${XROOTD_INC} ]
then
echo "ERROR: failed to setup xrootd"
exit 1
else
echo "XROOTD_INC: ${XROOTD_INC}"
fi

if [ "${shortqual}" = "nu" ]
then
    if [ -z ${POSTGRESQL_VERSION} ]
    then
       echo "ERROR: failed to setup postgresql"
       exit 1
    else
       echo "postgresql version: ${POSTGRESQL_VERSION}"
    fi

    if [ -z ${MYSQL_CLIENT_VERSION} ]
    then
       echo "ERROR: failed to setup mysql_client"
       exit 1
    else
       echo "mysql_client version: ${MYSQL_CLIENT_VERSION}"
    fi
fi

if [ -z ${PYTHONHOME} ]
then
   echo "ERROR: PYTHONHOME is undefined"
   echo "       something is very wrong"
   exit 1
fi

cd ${sourcedir} && rm -rf root
tar xf ${tardir}/${pkgtarfile} || ssi_die "Unable to untar source"

set_compiler_flags

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

cd ${sourcedir}/root-${pkgdotver}

# Apply patches
patches=(${patchdir}/fermi_root_6.patch \
  ${patchdir}/fermi_root_${pkgdotver//./_}.patch)
for patch in "${patches[@]}"; do
  [[ -r "${patch}" ]] || continue # Don't care if there isn't one this time.
  if grep -e '^\(---\|+++\) [ab]/' ${patch} >/dev/null 2>&1; then
    pval=1
  else
    pval=0
  fi
  patch ${DEBUG_PATCHES:+--verbose} -p${pval} < "${patch}" || ssi_die "application of patch ${patch} failed."
done

[[ -n "${DEBUG_PATCHES}" ]] && exit 1

cd ${blddir}

if [ ${xcc} = "gcc" ]
then
echo "Building with gcc: $(which gcc) and g++: $(which g++)" 1>&2
elif [ ${xcc} = "clang" ]
then
echo "Building with clang: $(which clang) and clang++: $(which clang++)" 1>&2
fi

if [ ${OS1} = "Darwin" ]
then
  darwin_xopts=(-Dcocoa=OFF -Dx11=On \
                -Dbonjour=OFF \
                -Dbuiltin_afterimage=ON \
                -Dbuiltin_ftgl=ON \
                -Dbuiltin_freetype=ON \
                -Dbuiltin_glew=ON \
                -Dbuiltin_pcre=ON \
                -Dbuiltin_lzma=ON \
 )
  # meant to be found only on El Capitan and newer.
  if [  -d /usr/local/opt/openssl ] && [[ "${OS%-*}" == d1[56] ]]; then
     darwin_xopts[${#darwin_xopts[@]}]=-DOPENSSL_ROOT_DIR=/usr/local/opt/openssl
  fi
  if [[ "${xcc}" = "gcc" ]]; then
    # GCC makes zero-size objects when __builtin_unreachable is used, which panics the OS X linker.
    export CXXFLAGS="-D__builtin_unreachable=__builtin_trap"
    export CFLAGS="-D__builtin_unreachable=__builtin_trap"
  fi
else
  makearch="linuxx8664${xcc}"
  if [[ $(ups flavor) == "Linux64bit+2.6-2.5" ]]; then # SLF 5.
    # LZMA only available through DAG -- not an option.
    extra_command="${extra_command} -Dbuiltin_lzma=ON"
  fi
fi

common_opts=(-DCMAKE_BUILD_TYPE=${buildtype} \
    -DCMAKE_INSTALL_PREFIX=${ROOTSYS} \
    "${compiler_specs[@]}" \
    -DROOT_ARCHITECTURE=${makearch} \
    -Dfail-on-missing=ON \
    -DCMAKE_IGNORE_PATH=/usr/local/lib\;/usr/local/include\;/usr/local/bin \
    -Dhttp=ON \
    -Dgdml=ON \
    -Dcxx14=ON \
    -Droofit=ON \
    -DCMAKE_PREFIX_PATH="${FFTW_FQ_DIR};${LIBXML2_FQ_DIR}" \
    -Dbuiltin_xrootd=OFF \
    -DXROOTD_ROOT_DIR="${XROOTDROOT}" \
    -Dminuit2=ON \
    -Dtbb=ON \
    -Dimt=ON \
    -Dbuiltin_tbb=OFF \
    -Dxft=OFF \
    -Dbuiltin_gsl=OFF \
    -Dsqlite=OFF \
    -Dtesting=OFF \
    -Dasimage=ON \
    -Dastiff=OFF \
    -Dgviz=OFF \
    -Dbonjour=OFF \
    -Dcastor=OFF \
    -Drfio=OFF \
    -Doracle=OFF \
    -Dodbc=OFF \
    -Dalien=OFF \
    -Dchirp=OFF \
    -Dpythia8=OFF \
    -Dmonalisa=OFF \
    -Dgfal=OFF \
    -Ddcache=OFF \
    -Dhdfs=OFF \
    -Dldap=OFF \
    -Dclarens=OFF \
    -Dglite=OFF \
    -Dpeac=OFF \
    -Dsapdb=OFF \
    -Dsrp=OFF \
    -Ddavix=OFF \
    -Dbuiltin_davix=OFF \
    -Dfitsio=OFF \
    -Dbuiltin_fitsio=OFF \
    ${darwin_xopts:+"${darwin_xopts[@]}"} \
)

if [[ -n "${DEBUG_LLVM}" ]] && [[ "${buildtype}" == "Debug" ]]; then
    common_opts[${#common_opts[@]}]="-DLLVM_BUILD_TYPE=${buildtype}"
    common_opts[${#common_opts[@]}]="-DLLVM_ENABLE_ASSERTIONS=ON"
fi

if [ "${shortqual}" = "nu" ]; then
  cmake "${common_opts[@]}" \
 -Dmathmore=ON \
 -Dgsl=ON \
 -Dgsl_shared=ON \
 -DGSL_DIR="${GSL_FQ_DIR}" \
 -Dmysql=ON \
 -DMYSQL_CONFIG_EXECUTABLE="${MYSQL_CLIENT_FQ_DIR}/bin/mysql_config" \
 -Dpgsql=ON \
 -DPOSTGRESQL_DIR="${POSTGRESQL_FQ_DIR}" \
 -Dpythia6=ON \
 -DPYTHIA6_DIR="${PYTHIA_FQ_DIR}" \
 ${sourcedir}/root-${pkgdotver}
else # Vanilla now has minuit2
  cmake "${common_opts[@]}" \
 -Dmathmore=OFF \
 -Dgsl=OFF \
 -Dmysql=OFF \
 -Dpgsql=OFF \
 -Dpythia6=OFF \
 ${sourcedir}/root-${pkgdotver}
fi

(( $? == 0 )) || ssi_die "cmake step failed."

make -j ${ncore} || ssi_die "make failed."
#make VERBOSE=t || ssi_die "make failed."

# now install in ROOTSYS
make install || ssi_die "make install failed."

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# need to fix paths:
# bin/root-config
# include/RConfigOptions.h
# include/compiledata.h
# etc/cling/llvm/Config/llvm-config.h
# etc/dictpch/allCppflags.txt

fixfile=/tmp/root-fix-${qualdir}-$$$$.sh.sh
rm -f $fixfile

echo "#!/bin/bash" > $fixfile
##echo " set -x " >>  $fixfile
echo "mv ${pkgdir}/bin/root-config ${pkgdir}/bin/root-config.bak" >> $fixfile
echo "cat ${pkgdir}/bin/root-config.bak | \\" >> $fixfile
echo "     sed -e 's%${LIBXML2_FQ_DIR}%\${LIBXML2_FQ_DIR}%g' \\" >> $fixfile
echo "         -e 's%${FFTW_FQ_DIR}%\${FFTW_FQ_DIR}%g' \\" >> $fixfile
echo "         -e 's%${XROOTDROOT}%\${XROOTDROOT}%g' \\" >> $fixfile
echo "         -e 's%${PYTHON_ROOT}%\${PYTHON_ROOT}%g' \\" >> $fixfile
if [ "${shortqual}" = "nu" ]
then
echo "         -e 's%${PYLIB}%\${PYLIB}%g' \\" >> $fixfile
echo "         -e 's%${POSTGRESQL_FQ_DIR}%\${POSTGRESQL_FQ_DIR}%g' \\" >> $fixfile
echo "         -e 's%${MYSQL_CLIENT_FQ_DIR}%\${MYSQL_CLIENT_FQ_DIR}%g' \\" >> $fixfile
fi
echo "     > ${pkgdir}/bin/root-config || exit 1" >> $fixfile
echo "chmod +x  ${pkgdir}/bin/root-config" >> $fixfile
echo "exit \$?" >> $fixfile
chmod +x $fixfile
$fixfile || ssi_die "Execution of fixfile script was unsuccessful."

rm -f $fixfile

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || ssi_die "setup failed"
echo "${package} is installed at ${ROOTSYS}"

if [ ${OS1} = "Darwin" ];then
 echo 
 echo "checking libHbook"
 otool -L ${ROOTSYS}/lib/libHbook.so
 echo
fi

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
